# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ...types import balance_list_params, balance_create_params, balance_update_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from .transactions import (
    TransactionsResource,
    AsyncTransactionsResource,
    TransactionsResourceWithRawResponse,
    AsyncTransactionsResourceWithRawResponse,
    TransactionsResourceWithStreamingResponse,
    AsyncTransactionsResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.balance import Balance
from .charge_schedules import (
    ChargeSchedulesResource,
    AsyncChargeSchedulesResource,
    ChargeSchedulesResourceWithRawResponse,
    AsyncChargeSchedulesResourceWithRawResponse,
    ChargeSchedulesResourceWithStreamingResponse,
    AsyncChargeSchedulesResourceWithStreamingResponse,
)
from .transaction_schedules import (
    TransactionSchedulesResource,
    AsyncTransactionSchedulesResource,
    TransactionSchedulesResourceWithRawResponse,
    AsyncTransactionSchedulesResourceWithRawResponse,
    TransactionSchedulesResourceWithStreamingResponse,
    AsyncTransactionSchedulesResourceWithStreamingResponse,
)

__all__ = ["BalancesResource", "AsyncBalancesResource"]


class BalancesResource(SyncAPIResource):
    @cached_property
    def transactions(self) -> TransactionsResource:
        return TransactionsResource(self._client)

    @cached_property
    def charge_schedules(self) -> ChargeSchedulesResource:
        return ChargeSchedulesResource(self._client)

    @cached_property
    def transaction_schedules(self) -> TransactionSchedulesResource:
        return TransactionSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> BalancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return BalancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BalancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return BalancesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        end_date: Union[str, datetime],
        name: str,
        start_date: Union[str, datetime],
        allow_overdraft: bool | Omit = omit,
        balance_draw_down_description: str | Omit = omit,
        consumptions_accounting_product_id: str | Omit = omit,
        contract_id: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        description: str | Omit = omit,
        fees_accounting_product_id: str | Omit = omit,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE",
                "USAGE",
                "MINIMUM_SPEND",
                "COUNTER_RUNNING_TOTAL_CHARGE",
                "COUNTER_ADJUSTMENT_DEBIT",
                "AD_HOC",
            ]
        ]
        | Omit = omit,
        overage_description: str | Omit = omit,
        overage_surcharge_percent: float | Omit = omit,
        product_ids: SequenceNotStr[str] | Omit = omit,
        rollover_amount: float | Omit = omit,
        rollover_end_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Create a new Balance for the given end customer Account.

        This endpoint allows you to create a new Balance for a specific end customer
        Account. The Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          code: Unique short code for the Balance.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          name: The official name for the Balance.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          allow_overdraft: Allow balance amounts to fall below zero. This feature is enabled on request.
              Please get in touch with m3ter Support or your m3ter contact if you would like
              it enabling for your organization(s).

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          consumptions_accounting_product_id: Product ID that any Balance Consumed line items will be attributed to for
              accounting purposes.(_Optional_)

          contract_id: The unique identifier (UUID) of a Contract on the Account that the Balance will
              be added to.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: A description of the Balance.

          fees_accounting_product_id: Product ID that any Balance Fees line items will be attributed to for accounting
              purposes.(_Optional_)

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`
              - `AD_HOC`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "allow_overdraft": allow_overdraft,
                    "balance_draw_down_description": balance_draw_down_description,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_create_params.BalanceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Retrieve a specific Balance.

        This endpoint returns the details of the specified Balance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        end_date: Union[str, datetime],
        name: str,
        start_date: Union[str, datetime],
        allow_overdraft: bool | Omit = omit,
        balance_draw_down_description: str | Omit = omit,
        consumptions_accounting_product_id: str | Omit = omit,
        contract_id: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        description: str | Omit = omit,
        fees_accounting_product_id: str | Omit = omit,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE",
                "USAGE",
                "MINIMUM_SPEND",
                "COUNTER_RUNNING_TOTAL_CHARGE",
                "COUNTER_ADJUSTMENT_DEBIT",
                "AD_HOC",
            ]
        ]
        | Omit = omit,
        overage_description: str | Omit = omit,
        overage_surcharge_percent: float | Omit = omit,
        product_ids: SequenceNotStr[str] | Omit = omit,
        rollover_amount: float | Omit = omit,
        rollover_end_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Update a specific Balance.

        This endpoint allows you to update the details of a specific Balance. The
        updated Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          code: Unique short code for the Balance.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          name: The official name for the Balance.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          allow_overdraft: Allow balance amounts to fall below zero. This feature is enabled on request.
              Please get in touch with m3ter Support or your m3ter contact if you would like
              it enabling for your organization(s).

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          consumptions_accounting_product_id: Product ID that any Balance Consumed line items will be attributed to for
              accounting purposes.(_Optional_)

          contract_id: The unique identifier (UUID) of a Contract on the Account that the Balance will
              be added to.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: A description of the Balance.

          fees_accounting_product_id: Product ID that any Balance Fees line items will be attributed to for accounting
              purposes.(_Optional_)

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`
              - `AD_HOC`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/balances/{id}",
            body=maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "allow_overdraft": allow_overdraft,
                    "balance_draw_down_description": balance_draw_down_description,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_update_params.BalanceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | Omit = omit,
        contract: str | Omit = omit,
        contract_id: str | Omit = omit,
        end_date_end: str | Omit = omit,
        end_date_start: str | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[Balance]:
        """
        Retrieve a list of all Balances for your Organization.

        This endpoint returns a list of all Balances associated with your organization.
        You can filter the Balances by the end customer's Account UUID and end dates,
        and paginate through them using the `pageSize` and `nextToken` parameters.

        **NOTE:** If a Balance has a rollover amount configured and you want to use the
        `endDateStart` or `endDateEnd` query parameters, the `rolloverEndDate` is used
        as the end date for the Balance.

        Args:
          account_id: The unique identifier (UUID) for the end customer's account.

          contract_id: Filter Balances by contract id. Use '' with accountId to fetch unlinked
              balances.

          end_date_end: Only include Balances with end dates earlier than this date. If a Balance has a
              rollover amount configured, then the `rolloverEndDate` will be used as the end
              date.

          end_date_start: Only include Balances with end dates equal to or later than this date. If a
              Balance has a rollover amount configured, then the `rolloverEndDate` will be
              used as the end date.

          ids: A list of unique identifiers (UUIDs) for specific Balances to retrieve.

          next_token: The `nextToken` for retrieving the next page of Balances. It is used to fetch
              the next page of Balances in a paginated list.

          page_size: The maximum number of Balances to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances",
            page=SyncCursor[Balance],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "contract": contract,
                        "contract_id": contract_id,
                        "end_date_end": end_date_end,
                        "end_date_start": end_date_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    balance_list_params.BalanceListParams,
                ),
            ),
            model=Balance,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Delete a specific Balance.

        This endpoint allows you to delete a specific Balance with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )


class AsyncBalancesResource(AsyncAPIResource):
    @cached_property
    def transactions(self) -> AsyncTransactionsResource:
        return AsyncTransactionsResource(self._client)

    @cached_property
    def charge_schedules(self) -> AsyncChargeSchedulesResource:
        return AsyncChargeSchedulesResource(self._client)

    @cached_property
    def transaction_schedules(self) -> AsyncTransactionSchedulesResource:
        return AsyncTransactionSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBalancesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBalancesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBalancesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncBalancesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        end_date: Union[str, datetime],
        name: str,
        start_date: Union[str, datetime],
        allow_overdraft: bool | Omit = omit,
        balance_draw_down_description: str | Omit = omit,
        consumptions_accounting_product_id: str | Omit = omit,
        contract_id: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        description: str | Omit = omit,
        fees_accounting_product_id: str | Omit = omit,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE",
                "USAGE",
                "MINIMUM_SPEND",
                "COUNTER_RUNNING_TOTAL_CHARGE",
                "COUNTER_ADJUSTMENT_DEBIT",
                "AD_HOC",
            ]
        ]
        | Omit = omit,
        overage_description: str | Omit = omit,
        overage_surcharge_percent: float | Omit = omit,
        product_ids: SequenceNotStr[str] | Omit = omit,
        rollover_amount: float | Omit = omit,
        rollover_end_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Create a new Balance for the given end customer Account.

        This endpoint allows you to create a new Balance for a specific end customer
        Account. The Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          code: Unique short code for the Balance.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          name: The official name for the Balance.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          allow_overdraft: Allow balance amounts to fall below zero. This feature is enabled on request.
              Please get in touch with m3ter Support or your m3ter contact if you would like
              it enabling for your organization(s).

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          consumptions_accounting_product_id: Product ID that any Balance Consumed line items will be attributed to for
              accounting purposes.(_Optional_)

          contract_id: The unique identifier (UUID) of a Contract on the Account that the Balance will
              be added to.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: A description of the Balance.

          fees_accounting_product_id: Product ID that any Balance Fees line items will be attributed to for accounting
              purposes.(_Optional_)

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`
              - `AD_HOC`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "allow_overdraft": allow_overdraft,
                    "balance_draw_down_description": balance_draw_down_description,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_create_params.BalanceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Retrieve a specific Balance.

        This endpoint returns the details of the specified Balance.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        account_id: str,
        code: str,
        currency: str,
        end_date: Union[str, datetime],
        name: str,
        start_date: Union[str, datetime],
        allow_overdraft: bool | Omit = omit,
        balance_draw_down_description: str | Omit = omit,
        consumptions_accounting_product_id: str | Omit = omit,
        contract_id: str | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        description: str | Omit = omit,
        fees_accounting_product_id: str | Omit = omit,
        line_item_types: List[
            Literal[
                "STANDING_CHARGE",
                "USAGE",
                "MINIMUM_SPEND",
                "COUNTER_RUNNING_TOTAL_CHARGE",
                "COUNTER_ADJUSTMENT_DEBIT",
                "AD_HOC",
            ]
        ]
        | Omit = omit,
        overage_description: str | Omit = omit,
        overage_surcharge_percent: float | Omit = omit,
        product_ids: SequenceNotStr[str] | Omit = omit,
        rollover_amount: float | Omit = omit,
        rollover_end_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Update a specific Balance.

        This endpoint allows you to update the details of a specific Balance. The
        updated Balance details should be provided in the request body.

        Args:
          account_id: The unique identifier (UUID) for the end customer Account.

          code: Unique short code for the Balance.

          currency: The currency code used for the Balance amount. For example: USD, GBP or EUR.

          end_date: The date _(in ISO 8601 format)_ after which the Balance will no longer be active
              for the Account.

              **Note:** You can use the `rolloverEndDate` request parameter to define an
              extended grace period for continued draw-down against the Balance if any amount
              remains when the specified `endDate` is reached.

          name: The official name for the Balance.

          start_date: The date _(in ISO 8601 format)_ when the Balance becomes active.

          allow_overdraft: Allow balance amounts to fall below zero. This feature is enabled on request.
              Please get in touch with m3ter Support or your m3ter contact if you would like
              it enabling for your organization(s).

          balance_draw_down_description: A description for the bill line items for draw-down charges against the Balance.
              _(Optional)._

          consumptions_accounting_product_id: Product ID that any Balance Consumed line items will be attributed to for
              accounting purposes.(_Optional_)

          contract_id: The unique identifier (UUID) of a Contract on the Account that the Balance will
              be added to.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          description: A description of the Balance.

          fees_accounting_product_id: Product ID that any Balance Fees line items will be attributed to for accounting
              purposes.(_Optional_)

          line_item_types: Specify the line item charge types that can draw-down at billing against the
              Balance amount. Options are:

              - `"MINIMUM_SPEND"`
              - `"STANDING_CHARGE"`
              - `"USAGE"`
              - `"COUNTER_RUNNING_TOTAL_CHARGE"`
              - `"COUNTER_ADJUSTMENT_DEBIT"`
              - `AD_HOC`

              **NOTE:** If no charge types are specified, by default _all types_ can draw-down
              against the Balance amount at billing.

          overage_description: A description for Bill line items overage charges.

          overage_surcharge_percent: Define a surcharge level, as a percentage of regular usage rating, applied to
              overages _(usage charges that exceed the Balance amount)_. For example, if the
              regular usage rate is $10 per unit of usage consumed and
              `overageSurchargePercent` is set at 10%, then any usage charged above the
              original Balance amount is charged at $11 per unit of usage.

          product_ids: Specify the Products whose consumption charges due at billing can be drawn-down
              against the Balance amount.

              **Note:** If you don't specify any Products for Balance draw-down, by default
              the consumption charges for any Product the Account consumes will be drawn-down
              against the Balance amount.

          rollover_amount: The maximum amount that can be carried over past the Balance end date for
              draw-down at billing if there is any unused Balance amount when the end date is
              reached. Works with `rolloverEndDate` to define the amount and duration of a
              Balance "grace period". _(Optional)_

              **Notes:**

              - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
                amount left over after the Balance end date is reached will be drawn-down
                against up to the specified `rolloverEndDate`.
              - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
                without entering a `rolloverEndDate`, you'll receive an error when trying to
                create or update the Balance.
              - If you don't want to grant any grace period for outstanding Balance amounts,
                then do not use `rolloverAmount` and `rolloverEndDate`.

          rollover_end_date: The end date _(in ISO 8601 format)_ for the grace period during which unused
              Balance amounts can be carried over and drawn-down against at billing.

              **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
              be carried over and made available for draw-down.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/balances/{id}",
            body=await async_maybe_transform(
                {
                    "account_id": account_id,
                    "code": code,
                    "currency": currency,
                    "end_date": end_date,
                    "name": name,
                    "start_date": start_date,
                    "allow_overdraft": allow_overdraft,
                    "balance_draw_down_description": balance_draw_down_description,
                    "consumptions_accounting_product_id": consumptions_accounting_product_id,
                    "contract_id": contract_id,
                    "custom_fields": custom_fields,
                    "description": description,
                    "fees_accounting_product_id": fees_accounting_product_id,
                    "line_item_types": line_item_types,
                    "overage_description": overage_description,
                    "overage_surcharge_percent": overage_surcharge_percent,
                    "product_ids": product_ids,
                    "rollover_amount": rollover_amount,
                    "rollover_end_date": rollover_end_date,
                    "version": version,
                },
                balance_update_params.BalanceUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        account_id: str | Omit = omit,
        contract: str | Omit = omit,
        contract_id: str | Omit = omit,
        end_date_end: str | Omit = omit,
        end_date_start: str | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Balance, AsyncCursor[Balance]]:
        """
        Retrieve a list of all Balances for your Organization.

        This endpoint returns a list of all Balances associated with your organization.
        You can filter the Balances by the end customer's Account UUID and end dates,
        and paginate through them using the `pageSize` and `nextToken` parameters.

        **NOTE:** If a Balance has a rollover amount configured and you want to use the
        `endDateStart` or `endDateEnd` query parameters, the `rolloverEndDate` is used
        as the end date for the Balance.

        Args:
          account_id: The unique identifier (UUID) for the end customer's account.

          contract_id: Filter Balances by contract id. Use '' with accountId to fetch unlinked
              balances.

          end_date_end: Only include Balances with end dates earlier than this date. If a Balance has a
              rollover amount configured, then the `rolloverEndDate` will be used as the end
              date.

          end_date_start: Only include Balances with end dates equal to or later than this date. If a
              Balance has a rollover amount configured, then the `rolloverEndDate` will be
              used as the end date.

          ids: A list of unique identifiers (UUIDs) for specific Balances to retrieve.

          next_token: The `nextToken` for retrieving the next page of Balances. It is used to fetch
              the next page of Balances in a paginated list.

          page_size: The maximum number of Balances to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances",
            page=AsyncCursor[Balance],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_id": account_id,
                        "contract": contract,
                        "contract_id": contract_id,
                        "end_date_end": end_date_end,
                        "end_date_start": end_date_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    balance_list_params.BalanceListParams,
                ),
            ),
            model=Balance,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Balance:
        """
        Delete a specific Balance.

        This endpoint allows you to delete a specific Balance with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/balances/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Balance,
        )


class BalancesResourceWithRawResponse:
    def __init__(self, balances: BalancesResource) -> None:
        self._balances = balances

        self.create = to_raw_response_wrapper(
            balances.create,
        )
        self.retrieve = to_raw_response_wrapper(
            balances.retrieve,
        )
        self.update = to_raw_response_wrapper(
            balances.update,
        )
        self.list = to_raw_response_wrapper(
            balances.list,
        )
        self.delete = to_raw_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithRawResponse:
        return TransactionsResourceWithRawResponse(self._balances.transactions)

    @cached_property
    def charge_schedules(self) -> ChargeSchedulesResourceWithRawResponse:
        return ChargeSchedulesResourceWithRawResponse(self._balances.charge_schedules)

    @cached_property
    def transaction_schedules(self) -> TransactionSchedulesResourceWithRawResponse:
        return TransactionSchedulesResourceWithRawResponse(self._balances.transaction_schedules)


class AsyncBalancesResourceWithRawResponse:
    def __init__(self, balances: AsyncBalancesResource) -> None:
        self._balances = balances

        self.create = async_to_raw_response_wrapper(
            balances.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            balances.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            balances.update,
        )
        self.list = async_to_raw_response_wrapper(
            balances.list,
        )
        self.delete = async_to_raw_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithRawResponse:
        return AsyncTransactionsResourceWithRawResponse(self._balances.transactions)

    @cached_property
    def charge_schedules(self) -> AsyncChargeSchedulesResourceWithRawResponse:
        return AsyncChargeSchedulesResourceWithRawResponse(self._balances.charge_schedules)

    @cached_property
    def transaction_schedules(self) -> AsyncTransactionSchedulesResourceWithRawResponse:
        return AsyncTransactionSchedulesResourceWithRawResponse(self._balances.transaction_schedules)


class BalancesResourceWithStreamingResponse:
    def __init__(self, balances: BalancesResource) -> None:
        self._balances = balances

        self.create = to_streamed_response_wrapper(
            balances.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            balances.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            balances.update,
        )
        self.list = to_streamed_response_wrapper(
            balances.list,
        )
        self.delete = to_streamed_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> TransactionsResourceWithStreamingResponse:
        return TransactionsResourceWithStreamingResponse(self._balances.transactions)

    @cached_property
    def charge_schedules(self) -> ChargeSchedulesResourceWithStreamingResponse:
        return ChargeSchedulesResourceWithStreamingResponse(self._balances.charge_schedules)

    @cached_property
    def transaction_schedules(self) -> TransactionSchedulesResourceWithStreamingResponse:
        return TransactionSchedulesResourceWithStreamingResponse(self._balances.transaction_schedules)


class AsyncBalancesResourceWithStreamingResponse:
    def __init__(self, balances: AsyncBalancesResource) -> None:
        self._balances = balances

        self.create = async_to_streamed_response_wrapper(
            balances.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            balances.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            balances.update,
        )
        self.list = async_to_streamed_response_wrapper(
            balances.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            balances.delete,
        )

    @cached_property
    def transactions(self) -> AsyncTransactionsResourceWithStreamingResponse:
        return AsyncTransactionsResourceWithStreamingResponse(self._balances.transactions)

    @cached_property
    def charge_schedules(self) -> AsyncChargeSchedulesResourceWithStreamingResponse:
        return AsyncChargeSchedulesResourceWithStreamingResponse(self._balances.charge_schedules)

    @cached_property
    def transaction_schedules(self) -> AsyncTransactionSchedulesResourceWithStreamingResponse:
        return AsyncTransactionSchedulesResourceWithStreamingResponse(self._balances.transaction_schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["CurrencyConversion"]

_CurrencyConversionReservedKeywords = TypedDict(
    "_CurrencyConversionReservedKeywords",
    {
        "from": str,
    },
    total=False,
)


class CurrencyConversion(_CurrencyConversionReservedKeywords, total=False):
    """
    An array of currency conversion rates from Bill currency to Organization currency. For example, if Account is billed in GBP and Organization is set to USD, Bill line items are calculated in GBP and then converted to USD using the defined rate.
    """

    to: Required[str]
    """Currency to convert to. For example: USD."""

    multiplier: float
    """Conversion rate between currencies."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union

import httpx

from ...types import (
    lookup_table_list_params,
    lookup_table_create_params,
    lookup_table_update_params,
    lookup_table_retrieve_params,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from .lookup_table_revisions import (
    LookupTableRevisionsResource,
    AsyncLookupTableRevisionsResource,
    LookupTableRevisionsResourceWithRawResponse,
    AsyncLookupTableRevisionsResourceWithRawResponse,
    LookupTableRevisionsResourceWithStreamingResponse,
    AsyncLookupTableRevisionsResourceWithStreamingResponse,
)
from ...types.lookup_table_response import LookupTableResponse
from .lookup_table_revision_data.lookup_table_revision_data import (
    LookupTableRevisionDataResource,
    AsyncLookupTableRevisionDataResource,
    LookupTableRevisionDataResourceWithRawResponse,
    AsyncLookupTableRevisionDataResourceWithRawResponse,
    LookupTableRevisionDataResourceWithStreamingResponse,
    AsyncLookupTableRevisionDataResourceWithStreamingResponse,
)

__all__ = ["LookupTablesResource", "AsyncLookupTablesResource"]


class LookupTablesResource(SyncAPIResource):
    @cached_property
    def lookup_table_revisions(self) -> LookupTableRevisionsResource:
        return LookupTableRevisionsResource(self._client)

    @cached_property
    def lookup_table_revision_data(self) -> LookupTableRevisionDataResource:
        return LookupTableRevisionDataResource(self._client)

    @cached_property
    def with_raw_response(self) -> LookupTablesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LookupTablesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LookupTablesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return LookupTablesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Create a new Lookup Table.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/lookuptables",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_create_params.LookupTableCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Retrieve a Lookup Table by UUID.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for the Lookup
              Tables returned, set `additional=activeRevision` in the query.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"additional": additional}, lookup_table_retrieve_params.LookupTableRetrieveParams
                ),
            ),
            cast_to=LookupTableResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Update the Lookup Table with the given UUID.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/lookuptables/{id}",
            body=maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_update_params.LookupTableUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[LookupTableResponse]:
        """
        Retrieve a list Lookup Tables created for the Organization:

        - Returned list can be filtered by Lookup Table `code` query parameter.
        - If you want to include any non-default fields for the returned Lookup Tables,
          use the additional query parameter to specify which you want to include in the
          response.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for each of the
              Lookup Tables returned, set `additional=activeRevision` in the query.

          codes: List of Lookup Table codes to retrieve.

          next_token: Token to supply for multi page retrievals.

          page_size: Number of Lookup Tables to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables",
            page=SyncCursor[LookupTableResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional": additional,
                        "codes": codes,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_list_params.LookupTableListParams,
                ),
            ),
            model=LookupTableResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Delete the Lookup Table with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )


class AsyncLookupTablesResource(AsyncAPIResource):
    @cached_property
    def lookup_table_revisions(self) -> AsyncLookupTableRevisionsResource:
        return AsyncLookupTableRevisionsResource(self._client)

    @cached_property
    def lookup_table_revision_data(self) -> AsyncLookupTableRevisionDataResource:
        return AsyncLookupTableRevisionDataResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncLookupTablesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLookupTablesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLookupTablesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncLookupTablesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Create a new Lookup Table.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/lookuptables",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_create_params.LookupTableCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Retrieve a Lookup Table by UUID.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for the Lookup
              Tables returned, set `additional=activeRevision` in the query.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"additional": additional}, lookup_table_retrieve_params.LookupTableRetrieveParams
                ),
            ),
            cast_to=LookupTableResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        code: str,
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Update the Lookup Table with the given UUID.

        Args:
          code: Code of the Lookup Table - unique short code used to identify the Lookup Table.

              **NOTE:** Code has a maximum length of 80 characters and must not contain
              non-printable or whitespace characters (except space), and cannot start/end with
              whitespace.

          name: Descriptive name for the Lookup Table.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/lookuptables/{id}",
            body=await async_maybe_transform(
                {
                    "code": code,
                    "name": name,
                    "custom_fields": custom_fields,
                    "version": version,
                },
                lookup_table_update_params.LookupTableUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        additional: SequenceNotStr[str] | Omit = omit,
        codes: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LookupTableResponse, AsyncCursor[LookupTableResponse]]:
        """
        Retrieve a list Lookup Tables created for the Organization:

        - Returned list can be filtered by Lookup Table `code` query parameter.
        - If you want to include any non-default fields for the returned Lookup Tables,
          use the additional query parameter to specify which you want to include in the
          response.

        Args:
          additional: Comma separated list of additional non-default fields to be included in the
              response. For example,if you want to include the active Revision for each of the
              Lookup Tables returned, set `additional=activeRevision` in the query.

          codes: List of Lookup Table codes to retrieve.

          next_token: Token to supply for multi page retrievals.

          page_size: Number of Lookup Tables to retrieve per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables",
            page=AsyncCursor[LookupTableResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "additional": additional,
                        "codes": codes,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_list_params.LookupTableListParams,
                ),
            ),
            model=LookupTableResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableResponse:
        """
        Delete the Lookup Table with the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/lookuptables/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableResponse,
        )


class LookupTablesResourceWithRawResponse:
    def __init__(self, lookup_tables: LookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = to_raw_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = to_raw_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = to_raw_response_wrapper(
            lookup_tables.update,
        )
        self.list = to_raw_response_wrapper(
            lookup_tables.list,
        )
        self.delete = to_raw_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> LookupTableRevisionsResourceWithRawResponse:
        return LookupTableRevisionsResourceWithRawResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> LookupTableRevisionDataResourceWithRawResponse:
        return LookupTableRevisionDataResourceWithRawResponse(self._lookup_tables.lookup_table_revision_data)


class AsyncLookupTablesResourceWithRawResponse:
    def __init__(self, lookup_tables: AsyncLookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = async_to_raw_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            lookup_tables.update,
        )
        self.list = async_to_raw_response_wrapper(
            lookup_tables.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> AsyncLookupTableRevisionsResourceWithRawResponse:
        return AsyncLookupTableRevisionsResourceWithRawResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> AsyncLookupTableRevisionDataResourceWithRawResponse:
        return AsyncLookupTableRevisionDataResourceWithRawResponse(self._lookup_tables.lookup_table_revision_data)


class LookupTablesResourceWithStreamingResponse:
    def __init__(self, lookup_tables: LookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = to_streamed_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            lookup_tables.update,
        )
        self.list = to_streamed_response_wrapper(
            lookup_tables.list,
        )
        self.delete = to_streamed_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> LookupTableRevisionsResourceWithStreamingResponse:
        return LookupTableRevisionsResourceWithStreamingResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> LookupTableRevisionDataResourceWithStreamingResponse:
        return LookupTableRevisionDataResourceWithStreamingResponse(self._lookup_tables.lookup_table_revision_data)


class AsyncLookupTablesResourceWithStreamingResponse:
    def __init__(self, lookup_tables: AsyncLookupTablesResource) -> None:
        self._lookup_tables = lookup_tables

        self.create = async_to_streamed_response_wrapper(
            lookup_tables.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            lookup_tables.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            lookup_tables.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lookup_tables.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lookup_tables.delete,
        )

    @cached_property
    def lookup_table_revisions(self) -> AsyncLookupTableRevisionsResourceWithStreamingResponse:
        return AsyncLookupTableRevisionsResourceWithStreamingResponse(self._lookup_tables.lookup_table_revisions)

    @cached_property
    def lookup_table_revision_data(self) -> AsyncLookupTableRevisionDataResourceWithStreamingResponse:
        return AsyncLookupTableRevisionDataResourceWithStreamingResponse(self._lookup_tables.lookup_table_revision_data)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["BillLatestByAccountParams"]


class BillLatestByAccountParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    additional: SequenceNotStr[str]
    """Comma separated list of additional fields."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .statement_jobs import (
    StatementJobsResource,
    AsyncStatementJobsResource,
    StatementJobsResourceWithRawResponse,
    AsyncStatementJobsResourceWithRawResponse,
    StatementJobsResourceWithStreamingResponse,
    AsyncStatementJobsResourceWithStreamingResponse,
)
from .statement_definitions import (
    StatementDefinitionsResource,
    AsyncStatementDefinitionsResource,
    StatementDefinitionsResourceWithRawResponse,
    AsyncStatementDefinitionsResourceWithRawResponse,
    StatementDefinitionsResourceWithStreamingResponse,
    AsyncStatementDefinitionsResourceWithStreamingResponse,
)
from ...types.object_url_response import ObjectURLResponse

__all__ = ["StatementsResource", "AsyncStatementsResource"]


class StatementsResource(SyncAPIResource):
    @cached_property
    def statement_jobs(self) -> StatementJobsResource:
        return StatementJobsResource(self._client)

    @cached_property
    def statement_definitions(self) -> StatementDefinitionsResource:
        return StatementDefinitionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> StatementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return StatementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> StatementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return StatementsResourceWithStreamingResponse(self)

    def create_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Generate a specific Bill Statement for the provided Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you then use
        with a `GET` call to obtain the Bill statement in CSV format.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    def get_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a specific Bill Statement for the given Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response includes a pre-signed `downloadUrl`, which must be used with a
        separate `GET` call to download the actual Bill Statement. This ensures secure
        access to the requested information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    def get_json(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a Bill Statement in JSON format for a given Bill ID.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you use with
        a `GET` call to obtain the Bill Statement.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/bills/{id}/statement/json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )


class AsyncStatementsResource(AsyncAPIResource):
    @cached_property
    def statement_jobs(self) -> AsyncStatementJobsResource:
        return AsyncStatementJobsResource(self._client)

    @cached_property
    def statement_definitions(self) -> AsyncStatementDefinitionsResource:
        return AsyncStatementDefinitionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncStatementsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncStatementsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncStatementsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncStatementsResourceWithStreamingResponse(self)

    async def create_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Generate a specific Bill Statement for the provided Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you then use
        with a `GET` call to obtain the Bill statement in CSV format.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    async def get_csv(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a specific Bill Statement for the given Bill UUID in CSV format.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response includes a pre-signed `downloadUrl`, which must be used with a
        separate `GET` call to download the actual Bill Statement. This ensures secure
        access to the requested information.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/{id}/statement/csv",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )

    async def get_json(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ObjectURLResponse:
        """
        Retrieve a Bill Statement in JSON format for a given Bill ID.

        Bill Statements are backing sheets to the invoices sent to your customers. Bill
        Statements provide a breakdown of the usage responsible for the usage charge
        line items shown on invoices.

        The response to this call returns a pre-signed `downloadUrl`, which you use with
        a `GET` call to obtain the Bill Statement.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/bills/{id}/statement/json",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ObjectURLResponse,
        )


class StatementsResourceWithRawResponse:
    def __init__(self, statements: StatementsResource) -> None:
        self._statements = statements

        self.create_csv = to_raw_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = to_raw_response_wrapper(
            statements.get_csv,
        )
        self.get_json = to_raw_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> StatementJobsResourceWithRawResponse:
        return StatementJobsResourceWithRawResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> StatementDefinitionsResourceWithRawResponse:
        return StatementDefinitionsResourceWithRawResponse(self._statements.statement_definitions)


class AsyncStatementsResourceWithRawResponse:
    def __init__(self, statements: AsyncStatementsResource) -> None:
        self._statements = statements

        self.create_csv = async_to_raw_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = async_to_raw_response_wrapper(
            statements.get_csv,
        )
        self.get_json = async_to_raw_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> AsyncStatementJobsResourceWithRawResponse:
        return AsyncStatementJobsResourceWithRawResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> AsyncStatementDefinitionsResourceWithRawResponse:
        return AsyncStatementDefinitionsResourceWithRawResponse(self._statements.statement_definitions)


class StatementsResourceWithStreamingResponse:
    def __init__(self, statements: StatementsResource) -> None:
        self._statements = statements

        self.create_csv = to_streamed_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = to_streamed_response_wrapper(
            statements.get_csv,
        )
        self.get_json = to_streamed_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> StatementJobsResourceWithStreamingResponse:
        return StatementJobsResourceWithStreamingResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> StatementDefinitionsResourceWithStreamingResponse:
        return StatementDefinitionsResourceWithStreamingResponse(self._statements.statement_definitions)


class AsyncStatementsResourceWithStreamingResponse:
    def __init__(self, statements: AsyncStatementsResource) -> None:
        self._statements = statements

        self.create_csv = async_to_streamed_response_wrapper(
            statements.create_csv,
        )
        self.get_csv = async_to_streamed_response_wrapper(
            statements.get_csv,
        )
        self.get_json = async_to_streamed_response_wrapper(
            statements.get_json,
        )

    @cached_property
    def statement_jobs(self) -> AsyncStatementJobsResourceWithStreamingResponse:
        return AsyncStatementJobsResourceWithStreamingResponse(self._statements.statement_jobs)

    @cached_property
    def statement_definitions(self) -> AsyncStatementDefinitionsResourceWithStreamingResponse:
        return AsyncStatementDefinitionsResourceWithStreamingResponse(self._statements.statement_definitions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["ChargeListParams"]


class ChargeListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Annotated[str, PropertyInfo(alias="accountId")]
    """List Charge items for the Account UUID"""

    bill_date: Annotated[Union[str, date], PropertyInfo(alias="billDate", format="iso8601")]
    """List Charge items for the Bill Date"""

    entity_id: Annotated[str, PropertyInfo(alias="entityId")]
    """List Charge items for the Entity UUID"""

    entity_type: Annotated[Literal["AD_HOC", "BALANCE"], PropertyInfo(alias="entityType")]
    """List Charge items for the EntityType"""

    ids: SequenceNotStr[str]
    """List of Charge UUIDs to retrieve"""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """nextToken for multi page retrievals"""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Charges to retrieve per page"""

    schedule_id: Annotated[str, PropertyInfo(alias="scheduleId")]
    """List Charge items for the Schedule UUID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import datetime

import httpx

from ..types import counter_pricing_list_params, counter_pricing_create_params, counter_pricing_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursor, AsyncCursor
from .._base_client import AsyncPaginator, make_request_options
from ..types.counter_pricing_response import CounterPricingResponse
from ..types.shared_params.pricing_band import PricingBand

__all__ = ["CounterPricingsResource", "AsyncCounterPricingsResource"]


class CounterPricingsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CounterPricingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return CounterPricingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CounterPricingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return CounterPricingsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        org_id: str | None = None,
        counter_id: str,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        code: str | Omit = omit,
        cumulative: bool | Omit = omit,
        description: str | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        plan_id: str | Omit = omit,
        plan_template_id: str | Omit = omit,
        pro_rate_adjustment_credit: bool | Omit = omit,
        pro_rate_adjustment_debit: bool | Omit = omit,
        pro_rate_running_total: bool | Omit = omit,
        running_total_bill_in_advance: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Create a new CounterPricing.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          counter_id: UUID of the Counter used to create the pricing.

          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          code: Unique short code for the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          pro_rate_adjustment_credit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment credits are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter adjustment credits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_adjustment_debit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment debits are prorated and are billed according
                to the number of days in billing period.

              - When **FALSE**, counter adjustment debits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_running_total: The default value is **TRUE**.

              - When **TRUE**, counter running total charges are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter running total charges are not prorated and are billed
                for the entire billing period.

              _(Optional)_.

          running_total_bill_in_advance: The default value is **TRUE**.

              - When **TRUE**, running totals are billed at the start of each billing period.

              - When **FALSE**, running totals are billed at the end of each billing period.

              _(Optional)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/counterpricings",
            body=maybe_transform(
                {
                    "counter_id": counter_id,
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "code": code,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "pro_rate_adjustment_credit": pro_rate_adjustment_credit,
                    "pro_rate_adjustment_debit": pro_rate_adjustment_debit,
                    "pro_rate_running_total": pro_rate_running_total,
                    "running_total_bill_in_advance": running_total_bill_in_advance,
                    "version": version,
                },
                counter_pricing_create_params.CounterPricingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Retrieve a CounterPricing for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/counterpricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        counter_id: str,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        code: str | Omit = omit,
        cumulative: bool | Omit = omit,
        description: str | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        plan_id: str | Omit = omit,
        plan_template_id: str | Omit = omit,
        pro_rate_adjustment_credit: bool | Omit = omit,
        pro_rate_adjustment_debit: bool | Omit = omit,
        pro_rate_running_total: bool | Omit = omit,
        running_total_bill_in_advance: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Update CounterPricing for the given UUID.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          counter_id: UUID of the Counter used to create the pricing.

          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          code: Unique short code for the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          pro_rate_adjustment_credit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment credits are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter adjustment credits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_adjustment_debit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment debits are prorated and are billed according
                to the number of days in billing period.

              - When **FALSE**, counter adjustment debits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_running_total: The default value is **TRUE**.

              - When **TRUE**, counter running total charges are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter running total charges are not prorated and are billed
                for the entire billing period.

              _(Optional)_.

          running_total_bill_in_advance: The default value is **TRUE**.

              - When **TRUE**, running totals are billed at the start of each billing period.

              - When **FALSE**, running totals are billed at the end of each billing period.

              _(Optional)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/counterpricings/{id}",
            body=maybe_transform(
                {
                    "counter_id": counter_id,
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "code": code,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "pro_rate_adjustment_credit": pro_rate_adjustment_credit,
                    "pro_rate_adjustment_debit": pro_rate_adjustment_debit,
                    "pro_rate_running_total": pro_rate_running_total,
                    "running_total_bill_in_advance": running_total_bill_in_advance,
                    "version": version,
                },
                counter_pricing_update_params.CounterPricingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date: str | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        plan_id: str | Omit = omit,
        plan_template_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[CounterPricingResponse]:
        """
        Retrieve a list of CounterPricing entities filtered by date, Plan ID, Plan
        Template ID, or CounterPricing ID.

        Args:
          date: Date on which to retrieve active CounterPricings.

          ids: List of CounterPricing IDs to retrieve.

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of CounterPricings to retrieve per page.

          plan_id: UUID of the Plan to retrieve CounterPricings for.

          plan_template_id: UUID of the Plan Template to retrieve CounterPricings for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/counterpricings",
            page=SyncCursor[CounterPricingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date": date,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan_id": plan_id,
                        "plan_template_id": plan_template_id,
                    },
                    counter_pricing_list_params.CounterPricingListParams,
                ),
            ),
            model=CounterPricingResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Delete a CounterPricing for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/counterpricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )


class AsyncCounterPricingsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCounterPricingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCounterPricingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCounterPricingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncCounterPricingsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        org_id: str | None = None,
        counter_id: str,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        code: str | Omit = omit,
        cumulative: bool | Omit = omit,
        description: str | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        plan_id: str | Omit = omit,
        plan_template_id: str | Omit = omit,
        pro_rate_adjustment_credit: bool | Omit = omit,
        pro_rate_adjustment_debit: bool | Omit = omit,
        pro_rate_running_total: bool | Omit = omit,
        running_total_bill_in_advance: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Create a new CounterPricing.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          counter_id: UUID of the Counter used to create the pricing.

          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          code: Unique short code for the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          pro_rate_adjustment_credit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment credits are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter adjustment credits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_adjustment_debit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment debits are prorated and are billed according
                to the number of days in billing period.

              - When **FALSE**, counter adjustment debits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_running_total: The default value is **TRUE**.

              - When **TRUE**, counter running total charges are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter running total charges are not prorated and are billed
                for the entire billing period.

              _(Optional)_.

          running_total_bill_in_advance: The default value is **TRUE**.

              - When **TRUE**, running totals are billed at the start of each billing period.

              - When **FALSE**, running totals are billed at the end of each billing period.

              _(Optional)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/counterpricings",
            body=await async_maybe_transform(
                {
                    "counter_id": counter_id,
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "code": code,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "pro_rate_adjustment_credit": pro_rate_adjustment_credit,
                    "pro_rate_adjustment_debit": pro_rate_adjustment_debit,
                    "pro_rate_running_total": pro_rate_running_total,
                    "running_total_bill_in_advance": running_total_bill_in_advance,
                    "version": version,
                },
                counter_pricing_create_params.CounterPricingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Retrieve a CounterPricing for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/counterpricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        counter_id: str,
        pricing_bands: Iterable[PricingBand],
        start_date: Union[str, datetime],
        accounting_product_id: str | Omit = omit,
        code: str | Omit = omit,
        cumulative: bool | Omit = omit,
        description: str | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        plan_id: str | Omit = omit,
        plan_template_id: str | Omit = omit,
        pro_rate_adjustment_credit: bool | Omit = omit,
        pro_rate_adjustment_debit: bool | Omit = omit,
        pro_rate_running_total: bool | Omit = omit,
        running_total_bill_in_advance: bool | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Update CounterPricing for the given UUID.

        **Note:** Either `planId` or `planTemplateId` request parameters are required
        for this call to be valid. If you omit both, then you will receive a validation
        error.

        Args:
          counter_id: UUID of the Counter used to create the pricing.

          pricing_bands

          start_date: The start date _(in ISO-8601 format)_ for when the Pricing starts to be active
              for the Plan of Plan Template._(Required)_

          accounting_product_id: Optional Product ID this Pricing should be attributed to for accounting purposes

          code: Unique short code for the Pricing.

          cumulative: Controls whether or not charge rates under a set of pricing bands configured for
              a Pricing are applied according to each separate band or at the highest band
              reached.

              _(Optional)_. The default value is **FALSE**.

              - When TRUE, at billing charge rates are applied according to each separate
                band.

              - When FALSE, at billing charge rates are applied according to highest band
                reached.

              **NOTE:** Use the `cumulative` parameter to create the type of Pricing you
              require. For example, for Tiered Pricing set to **TRUE**; for Volume Pricing,
              set to **FALSE**.

          description: Displayed on Bill line items.

          end_date: The end date _(in ISO-8601 format)_ for when the Pricing ceases to be active for
              the Plan or Plan Template.

              _(Optional)_ If not specified, the Pricing remains active indefinitely.

          plan_id: UUID of the Plan the Pricing is created for.

          plan_template_id: UUID of the Plan Template the Pricing is created for.

          pro_rate_adjustment_credit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment credits are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter adjustment credits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_adjustment_debit: The default value is **TRUE**.

              - When **TRUE**, counter adjustment debits are prorated and are billed according
                to the number of days in billing period.

              - When **FALSE**, counter adjustment debits are not prorated and are billed for
                the entire billing period.

              _(Optional)_.

          pro_rate_running_total: The default value is **TRUE**.

              - When **TRUE**, counter running total charges are prorated and are billed
                according to the number of days in billing period.

              - When **FALSE**, counter running total charges are not prorated and are billed
                for the entire billing period.

              _(Optional)_.

          running_total_bill_in_advance: The default value is **TRUE**.

              - When **TRUE**, running totals are billed at the start of each billing period.

              - When **FALSE**, running totals are billed at the end of each billing period.

              _(Optional)_.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/counterpricings/{id}",
            body=await async_maybe_transform(
                {
                    "counter_id": counter_id,
                    "pricing_bands": pricing_bands,
                    "start_date": start_date,
                    "accounting_product_id": accounting_product_id,
                    "code": code,
                    "cumulative": cumulative,
                    "description": description,
                    "end_date": end_date,
                    "plan_id": plan_id,
                    "plan_template_id": plan_template_id,
                    "pro_rate_adjustment_credit": pro_rate_adjustment_credit,
                    "pro_rate_adjustment_debit": pro_rate_adjustment_debit,
                    "pro_rate_running_total": pro_rate_running_total,
                    "running_total_bill_in_advance": running_total_bill_in_advance,
                    "version": version,
                },
                counter_pricing_update_params.CounterPricingUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date: str | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        plan_id: str | Omit = omit,
        plan_template_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CounterPricingResponse, AsyncCursor[CounterPricingResponse]]:
        """
        Retrieve a list of CounterPricing entities filtered by date, Plan ID, Plan
        Template ID, or CounterPricing ID.

        Args:
          date: Date on which to retrieve active CounterPricings.

          ids: List of CounterPricing IDs to retrieve.

          next_token: `nextToken` for multi page retrievals.

          page_size: Number of CounterPricings to retrieve per page.

          plan_id: UUID of the Plan to retrieve CounterPricings for.

          plan_template_id: UUID of the Plan Template to retrieve CounterPricings for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/counterpricings",
            page=AsyncCursor[CounterPricingResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date": date,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "plan_id": plan_id,
                        "plan_template_id": plan_template_id,
                    },
                    counter_pricing_list_params.CounterPricingListParams,
                ),
            ),
            model=CounterPricingResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CounterPricingResponse:
        """
        Delete a CounterPricing for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/counterpricings/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CounterPricingResponse,
        )


class CounterPricingsResourceWithRawResponse:
    def __init__(self, counter_pricings: CounterPricingsResource) -> None:
        self._counter_pricings = counter_pricings

        self.create = to_raw_response_wrapper(
            counter_pricings.create,
        )
        self.retrieve = to_raw_response_wrapper(
            counter_pricings.retrieve,
        )
        self.update = to_raw_response_wrapper(
            counter_pricings.update,
        )
        self.list = to_raw_response_wrapper(
            counter_pricings.list,
        )
        self.delete = to_raw_response_wrapper(
            counter_pricings.delete,
        )


class AsyncCounterPricingsResourceWithRawResponse:
    def __init__(self, counter_pricings: AsyncCounterPricingsResource) -> None:
        self._counter_pricings = counter_pricings

        self.create = async_to_raw_response_wrapper(
            counter_pricings.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            counter_pricings.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            counter_pricings.update,
        )
        self.list = async_to_raw_response_wrapper(
            counter_pricings.list,
        )
        self.delete = async_to_raw_response_wrapper(
            counter_pricings.delete,
        )


class CounterPricingsResourceWithStreamingResponse:
    def __init__(self, counter_pricings: CounterPricingsResource) -> None:
        self._counter_pricings = counter_pricings

        self.create = to_streamed_response_wrapper(
            counter_pricings.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            counter_pricings.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            counter_pricings.update,
        )
        self.list = to_streamed_response_wrapper(
            counter_pricings.list,
        )
        self.delete = to_streamed_response_wrapper(
            counter_pricings.delete,
        )


class AsyncCounterPricingsResourceWithStreamingResponse:
    def __init__(self, counter_pricings: AsyncCounterPricingsResource) -> None:
        self._counter_pricings = counter_pricings

        self.create = async_to_streamed_response_wrapper(
            counter_pricings.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            counter_pricings.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            counter_pricings.update,
        )
        self.list = async_to_streamed_response_wrapper(
            counter_pricings.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            counter_pricings.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .lookup_tables import (
    LookupTablesResource,
    AsyncLookupTablesResource,
    LookupTablesResourceWithRawResponse,
    AsyncLookupTablesResourceWithRawResponse,
    LookupTablesResourceWithStreamingResponse,
    AsyncLookupTablesResourceWithStreamingResponse,
)
from .lookup_table_revisions import (
    LookupTableRevisionsResource,
    AsyncLookupTableRevisionsResource,
    LookupTableRevisionsResourceWithRawResponse,
    AsyncLookupTableRevisionsResourceWithRawResponse,
    LookupTableRevisionsResourceWithStreamingResponse,
    AsyncLookupTableRevisionsResourceWithStreamingResponse,
)
from .lookup_table_revision_data import (
    LookupTableRevisionDataResource,
    AsyncLookupTableRevisionDataResource,
    LookupTableRevisionDataResourceWithRawResponse,
    AsyncLookupTableRevisionDataResourceWithRawResponse,
    LookupTableRevisionDataResourceWithStreamingResponse,
    AsyncLookupTableRevisionDataResourceWithStreamingResponse,
)

__all__ = [
    "LookupTableRevisionsResource",
    "AsyncLookupTableRevisionsResource",
    "LookupTableRevisionsResourceWithRawResponse",
    "AsyncLookupTableRevisionsResourceWithRawResponse",
    "LookupTableRevisionsResourceWithStreamingResponse",
    "AsyncLookupTableRevisionsResourceWithStreamingResponse",
    "LookupTableRevisionDataResource",
    "AsyncLookupTableRevisionDataResource",
    "LookupTableRevisionDataResourceWithRawResponse",
    "AsyncLookupTableRevisionDataResourceWithRawResponse",
    "LookupTableRevisionDataResourceWithStreamingResponse",
    "AsyncLookupTableRevisionDataResourceWithStreamingResponse",
    "LookupTablesResource",
    "AsyncLookupTablesResource",
    "LookupTablesResourceWithRawResponse",
    "AsyncLookupTablesResourceWithRawResponse",
    "LookupTablesResourceWithStreamingResponse",
    "AsyncLookupTablesResourceWithStreamingResponse",
]
